'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var cssElementQueries = require('css-element-queries');

var Centered = function (props) { return (React.createElement("div", { className: "spaces-centered" }, props.children)); };
var CenteredVertically = function (props) { return (React.createElement("div", { className: "spaces-centered-vertically" }, props.children)); };

var ClearFix = function (props) { return (React.createElement("div", { className: "spaces-clearfix" }, props.children)); };

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var ResizeType;
(function (ResizeType) {
    ResizeType["Left"] = "resize-left";
    ResizeType["Right"] = "resize-right";
    ResizeType["Top"] = "resize-top";
    ResizeType["Bottom"] = "resize-bottom";
})(ResizeType || (ResizeType = {}));
var Resizable = function (props) {
    var _a = React.useState(0), adjustedSize = _a[0], setAdjustedSize = _a[1];
    var startResize = function (e) {
        var originalMouseX = props.type === ResizeType.Left ? e.pageX + adjustedSize : e.pageX - adjustedSize;
        var originalMouseY = props.type === ResizeType.Top ? e.pageY + adjustedSize : e.pageY - adjustedSize;
        var resize = function (e) {
            var adjustmentX = Math.min(Math.max(props.type === ResizeType.Left ? originalMouseX - e.pageX : e.pageX - originalMouseX, props.minimumAdjust), props.maximumAdjust || 999999);
            var adjustmentY = Math.min(Math.max(props.type === ResizeType.Top ? originalMouseY - e.pageY : e.pageY - originalMouseY, props.minimumAdjust), props.maximumAdjust || 999999);
            var adjustment = props.type === ResizeType.Left || props.type === ResizeType.Right ? adjustmentX : adjustmentY;
            if (adjustment !== adjustedSize) {
                setAdjustedSize(adjustment);
                props.onResize(adjustment);
            }
        };
        var stopResize = function () { return window.removeEventListener('mousemove', resize); };
        e.preventDefault();
        window.addEventListener('mousemove', resize);
        window.addEventListener('mouseup', stopResize);
    };
    return (React.createElement("div", { style: { width: props.width, height: props.height }, className: "spaces-resize-handle " + props.type, onMouseDown: startResize }));
};

(function (AnchorType) {
    AnchorType["Left"] = "anchor-left";
    AnchorType["Right"] = "anchor-right";
    AnchorType["Top"] = "anchor-top";
    AnchorType["Bottom"] = "anchor-bottom";
})(exports.AnchorType || (exports.AnchorType = {}));
var AnchorTypes = [
    exports.AnchorType.Left,
    exports.AnchorType.Right,
    exports.AnchorType.Bottom,
    exports.AnchorType.Top
];
var AnchorToResizeTypeMap = {
    "anchor-left": ResizeType.Right,
    "anchor-right": ResizeType.Left,
    "anchor-bottom": ResizeType.Top,
    "anchor-top": ResizeType.Bottom
};
(function (CenterType) {
    CenterType[CenterType["None"] = 0] = "None";
    CenterType[CenterType["Vertical"] = 1] = "Vertical";
    CenterType[CenterType["HorizontalVertical"] = 2] = "HorizontalVertical";
})(exports.CenterType || (exports.CenterType = {}));

var SpaceContext = React.createContext(null);
var SpaceLayerContext = React.createContext(null);
var SpaceInfoContext = React.createContext(null);

var getSizeString = function (size) { return typeof (size) === "string" ? size : size + "px"; };
var isFilledSpace = function (props) { return !props.anchor; };
var isHorizontalSpace = function (props) {
    return props.anchor && (props.anchor === exports.AnchorType.Left || props.anchor === exports.AnchorType.Right);
};
var isVerticalSpace = function (props) {
    return props.anchor && (props.anchor === exports.AnchorType.Top || props.anchor === exports.AnchorType.Bottom);
};
var uuid = function () {
    return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (char) {
        var random = Math.random() * 16 | 0;
        var value = char === "x" ? random : (random % 4 + 8);
        return value.toString(16);
    });
};
var initialState = function (props) { return ({
    id: props.id || uuid(),
    currentWidth: 0,
    currentHeight: 0,
    adjustedSize: 0,
    adjustedLeft: 0,
    adjustedTop: 0,
    spaceTakers: [],
    parsedSize: typeof props.anchorSize === "string" ? 0 : props.anchorSize,
    left: props.anchor && props.anchor !== exports.AnchorType.Right ? props.left || 0 : props.left,
    top: props.anchor && props.anchor !== exports.AnchorType.Bottom ? props.top || 0 : props.top,
    right: props.anchor && props.anchor !== exports.AnchorType.Left ? props.right || 0 : props.right,
    bottom: props.anchor && props.anchor !== exports.AnchorType.Top ? props.bottom || 0 : props.bottom,
    width: props.anchor && isHorizontalSpace(props) ? props.anchorSize || 0 : props.width,
    height: props.anchor && isVerticalSpace(props) ? props.anchorSize || 0 : props.height,
    debug: props.debug !== undefined ? props.debug : false,
}); };
var createContext = function (state, setState, parent, zIndex) {
    var context = {
        debug: parent ? (parent.debug ? true : state.debug) : state.debug,
        level: parent ? parent.level + 1 : 0,
        zIndex: zIndex,
        width: state.currentWidth,
        height: state.currentHeight,
        spaceTakers: state.spaceTakers,
        registerSpaceTaker: function (spaceTaker) {
            var existing = state.spaceTakers.find(function (t) { return t.id === spaceTaker.id; });
            if (!existing) {
                setState({ spaceTakers: state.spaceTakers.concat([spaceTaker]) });
            }
            else {
                existing.order = spaceTaker.order;
                existing.anchorType = spaceTaker.anchorType;
                existing.size = spaceTaker.size;
            }
        },
        removeSpaceTaker: function (id) {
            return setState({ spaceTakers: state.spaceTakers.filter(function (t) { return t.id !== id; }) });
        },
        updateSpaceTakerAdjustedSize: function (id, adjustedSize) {
            return setState({ spaceTakers: state.spaceTakers.map(function (t) { return t.id === id ? __assign({}, t, { adjustedSize: adjustedSize }) : t; }) });
        },
        updateDebug: function (id, debug) {
            return setState({ spaceTakers: state.spaceTakers.map(function (t) { return t.id === id ? __assign({}, t, { debug: debug }) : t; }) });
        },
        startDrag: function (e) {
            var originalMouseX = e.pageX - state.adjustedLeft;
            var originalMouseY = e.pageY - state.adjustedTop;
            var drag = function (e) { return setState({ adjustedLeft: e.pageX - originalMouseX, adjustedTop: e.pageY - originalMouseY }); };
            var stopDrag = function () { return window.removeEventListener('mousemove', drag); };
            e.preventDefault();
            window.addEventListener('mousemove', drag);
            window.addEventListener('mouseup', stopDrag);
        }
    };
    return context;
};
var applyResize = function (props, state, setState, parentContext, handleSize) {
    if (parentContext && props.anchor && props.resizable) {
        var resizeType = AnchorToResizeTypeMap[props.anchor];
        var resizeHandleWidth = resizeType === ResizeType.Left || resizeType === ResizeType.Right ? props.handleSize || 5 : undefined;
        var resizeHandleHeight = resizeType === ResizeType.Top || resizeType === ResizeType.Bottom ? props.handleSize || 5 : undefined;
        return {
            resizeHandle: React.createElement(Resizable, { type: resizeType, width: resizeHandleWidth, height: resizeHandleHeight, minimumAdjust: (props.minimumSize || 20) - (state.parsedSize || 0), maximumAdjust: props.maximumSize ? (props.maximumSize - (state.parsedSize || 0)) : undefined, onResize: function (adjustedSize) {
                    setState({ adjustedSize: adjustedSize });
                    parentContext.updateSpaceTakerAdjustedSize(state.id, adjustedSize);
                } }),
            resizeType: resizeType
        };
    }
    return { resizeHandle: null, resizeType: null };
};

var useSpace = function (props, divElementRef) {
    var _a = React.useState(initialState(props)), state = _a[0], changeState = _a[1];
    var resizeSensor = React.useRef();
    var onRemove = React.useRef();
    var setState = function (stateDelta) { return changeState(function (prev) { return (__assign({}, prev, stateDelta)); }); };
    // Deal with property changes to size / anchoring 
    React.useEffect(function () {
        setState({
            parsedSize: typeof props.anchorSize === "string" ? 0 : props.anchorSize,
            left: props.anchor !== exports.AnchorType.Right ? props.left || 0 : undefined,
            top: props.anchor !== exports.AnchorType.Bottom ? props.top || 0 : undefined,
            right: props.anchor !== exports.AnchorType.Left ? props.right || 0 : undefined,
            bottom: props.anchor !== exports.AnchorType.Top ? props.bottom || 0 : undefined,
            width: isHorizontalSpace(props) ? props.anchorSize || 0 : props.width,
            height: isVerticalSpace(props) ? props.anchorSize || 0 : props.height,
            debug: props.debug !== undefined ? props.debug : false,
        });
    }, [props.zIndex, props.left, props.top, props.bottom, props.right, props.width, props.height, props.anchor, props.anchorSize, props.debug]);
    // Setup / cleanup
    React.useEffect(function () {
        if (divElementRef.current) {
            if (props.trackSize) {
                resizeSensor.current = new cssElementQueries.ResizeSensor(divElementRef.current, function (size) { return setState({ currentWidth: size.width, currentHeight: size.height }); });
            }
        }
        var cleanup = function () {
            resizeSensor.current && resizeSensor.current.detach();
            onRemove.current && onRemove.current();
        };
        return cleanup;
    }, []);
    React.useEffect(function () {
        if (divElementRef.current && !state.parsedSize) {
            var currentRect = divElementRef.current.getBoundingClientRect();
            setState({
                parsedSize: (isHorizontalSpace(props) ? currentRect.width : currentRect.height),
                currentWidth: currentRect.width,
                currentHeight: currentRect.height
            });
        }
    });
    var parentContext = React.useContext(SpaceContext);
    var currentZIndex = props.zIndex || React.useContext(SpaceLayerContext) || 0;
    var outerStyle = {
        left: (state.left !== undefined ? "calc(" + getSizeString(state.left || 0) + (state.adjustedLeft ? " + " + state.adjustedLeft + "px" : "") + ")" : undefined),
        top: (state.top !== undefined ? "calc(" + getSizeString(state.top || 0) + (state.adjustedTop ? " + " + state.adjustedTop + "px" : "") + ")" : undefined),
        right: (state.right !== undefined ? "calc(" + getSizeString(state.right || 0) + (state.adjustedLeft ? " - " + state.adjustedLeft + "px" : "") + ")" : undefined),
        bottom: (state.bottom !== undefined ? "calc(" + getSizeString(state.bottom || 0) + (state.adjustedTop ? " - " + state.adjustedTop + "px" : "") + ")" : undefined),
        width: isHorizontalSpace(props) ? "calc(" + getSizeString(props.anchorSize || 0) + " + " + state.adjustedSize + "px)" : state.width,
        height: isVerticalSpace(props) ? "calc(" + getSizeString(props.anchorSize || 0) + " + " + state.adjustedSize + "px)" : state.height,
        zIndex: currentZIndex,
    };
    if (parentContext) {
        onRemove.current = function () {
            console.log("Removing " + state.id);
            parentContext.removeSpaceTaker(state.id);
        };
        var adjustedTop = [];
        var adjustedLeft = [];
        var adjustedRight = [];
        var adjustedBottom = [];
        var _loop_1 = function (at) {
            var spaceTakers = parentContext.spaceTakers
                .filter(function (t) { return t.zIndex === currentZIndex && t.anchorType === AnchorTypes[at]; })
                .sort(function (a, b) { return a.order - b.order; });
            for (var i = 0; i < spaceTakers.length; i++) {
                var t = spaceTakers[i];
                if (t.id !== state.id) {
                    var adjustedSize = t.adjustedSize !== 0 ? " + " + t.adjustedSize + "px" : "";
                    if (isFilledSpace(props)) {
                        if (t.anchorType === exports.AnchorType.Top) {
                            adjustedTop.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Left) {
                            adjustedLeft.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Bottom) {
                            adjustedBottom.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Right) {
                            adjustedRight.push("" + getSizeString(t.size) + adjustedSize);
                        }
                    }
                    else {
                        if (t.anchorType === exports.AnchorType.Top && outerStyle.top !== undefined) {
                            adjustedTop.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Left && outerStyle.left !== undefined) {
                            adjustedLeft.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Bottom && outerStyle.bottom !== undefined) {
                            adjustedBottom.push("" + getSizeString(t.size) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Right && outerStyle.right !== undefined) {
                            adjustedRight.push("" + getSizeString(t.size) + adjustedSize);
                        }
                    }
                }
                else {
                    break;
                }
            }
        };
        for (var at = 0; at < AnchorTypes.length; at++) {
            _loop_1(at);
        }
        [
            { adjusted: adjustedTop, setter: function (value) { return outerStyle.top = value; } },
            { adjusted: adjustedBottom, setter: function (value) { return outerStyle.bottom = value; } },
            { adjusted: adjustedLeft, setter: function (value) { return outerStyle.left = value; } },
            { adjusted: adjustedRight, setter: function (value) { return outerStyle.right = value; } }
        ].map(function (x) { return x.adjusted.length > 0 && x.setter("calc(" + x.adjusted.join(" + ") + ")"); });
        props.anchor && parentContext.registerSpaceTaker({
            id: state.id,
            zIndex: currentZIndex,
            order: props.order || 1,
            anchorType: props.anchor,
            size: props.anchorSize || 0,
            adjustedSize: 0
        });
    }
    var currentContext = createContext(state, setState, parentContext, currentZIndex);
    var handleSize = props.handleSize || 5;
    var overlayHandle = props.overlayHandle !== undefined ? props.overlayHandle : true;
    var resize = applyResize(props, state, setState, parentContext);
    var innerStyle = __assign({}, props.style, {
        left: resize.resizeType === ResizeType.Left && !overlayHandle ? handleSize : undefined,
        top: resize.resizeType === ResizeType.Top && !overlayHandle ? handleSize : undefined,
        right: resize.resizeType === ResizeType.Right && !overlayHandle ? handleSize : undefined,
        bottom: resize.resizeType === ResizeType.Bottom && !overlayHandle ? handleSize : undefined
    });
    var debug = parentContext ? parentContext.debug : 
        props.debug !== undefined ? props.debug : false;
    var outerClasses = [
        "spaces-space",
        props.anchor || '',
        resize.resizeType || '',
        props.scrollable ? "scrollable" : '',
        props.className ? props.className + "-container" : '',
        debug ? 'debug' : ''
    ];
    var innerClasses = [
        "spaces-space-inner",
        props.className ? props.className : ''
    ];
    return {
        currentContext: currentContext,
        resizeHandle: resize.resizeHandle,
        outerStyle: outerStyle,
        outerClasses: outerClasses,
        innerStyle: innerStyle,
        innerClasses: innerClasses,
        currentWidth: state.currentWidth,
        currentHeight: state.currentHeight
    };
};
var useParentSpace = function () {
    var parentContext = React.useContext(SpaceContext);
    return {
        startDrag: parentContext ? parentContext.startDrag : function () { return null; }
    };
};

var Fill = function (props) { return React.createElement(SpaceInternal, __assign({}, props)); };
var Top = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Top, anchorSize: props.size })); };
var Left = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Left, anchorSize: props.size })); };
var Bottom = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Bottom, anchorSize: props.size })); };
var Right = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Right, anchorSize: props.size })); };
var TopResizable = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Top, anchorSize: props.size, resizable: true })); };
var LeftResizable = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Left, anchorSize: props.size, resizable: true })); };
var BottomResizable = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Bottom, anchorSize: props.size, resizable: true })); };
var RightResizable = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Right, anchorSize: props.size, resizable: true })); };
var Positioned = function (props) { return React.createElement(SpaceInternal, __assign({}, props)); };
var SpaceInternal = function (props) {
    var divElementRef = React.useRef();
    var _a = useSpace(props, divElementRef), currentContext = _a.currentContext, outerStyle = _a.outerStyle, outerClasses = _a.outerClasses, innerStyle = _a.innerStyle, innerClasses = _a.innerClasses, resizeHandle = _a.resizeHandle, currentWidth = _a.currentWidth, currentHeight = _a.currentHeight;
    var children = props.children;
    if (props.centerContent === exports.CenterType.Vertical) {
        children = React.createElement(CenteredVertically, null, children);
    }
    else if (props.centerContent === exports.CenterType.HorizontalVertical) {
        children = React.createElement(Centered, null, children);
    }
    return (React.createElement(SpaceContext.Provider, { value: currentContext },
        React.createElement(SpaceInfoContext.Provider, { value: { width: Math.floor(currentWidth), height: Math.floor(currentHeight) } }, React.createElement(props.as || 'div', {
            id: props.id,
            ref: divElementRef,
            className: outerClasses.join(' '),
            style: outerStyle,
            onClick: props.onClick,
            onMouseDown: props.onMouseDown,
            onMouseEnter: props.onMouseEnter,
            onMouseLeave: props.onMouseLeave
        }, React.createElement(React.Fragment, null,
            resizeHandle,
            React.createElement("div", { className: innerClasses.join(' '), style: innerStyle }, children))))));
};

var Fixed = function (props) {
    var style = __assign({}, props.style, {
        width: props.width,
        height: props.height
    });
    return (React.createElement("div", { className: "spaces-fixedsize-layout" + (props.className ? " " + props.className : ""), style: style },
        React.createElement(Fill, null, props.children)));
};

var ViewPort = function (props) { return (React.createElement("div", { className: "spaces-fullpage-layout" + (props.className ? " " + props.className : ""), style: {
        left: props.left || 0,
        top: props.top || 0,
        right: props.right || 0,
        bottom: props.bottom || 0
    } },
    React.createElement(Fill, null, props.children))); };

var Layer = function (props) {
    return React.createElement(SpaceLayerContext.Provider, { value: props.zIndex }, props.children);
};

var Info = function (props) { return (React.createElement(SpaceInfoContext.Consumer, null, function (info) { return props.children(info); })); };

exports.Bottom = Bottom;
exports.BottomResizable = BottomResizable;
exports.Centered = Centered;
exports.CenteredVertically = CenteredVertically;
exports.ClearFix = ClearFix;
exports.Fill = Fill;
exports.Fixed = Fixed;
exports.Info = Info;
exports.Layer = Layer;
exports.Left = Left;
exports.LeftResizable = LeftResizable;
exports.Positioned = Positioned;
exports.Right = Right;
exports.RightResizable = RightResizable;
exports.Top = Top;
exports.TopResizable = TopResizable;
exports.ViewPort = ViewPort;
exports.useParentSpace = useParentSpace;
//# sourceMappingURL=server.js.map
